function dsi_oceanwind(varargin)
% DataSetInterface for the OceanWind 
%
% Meri Sheremet
% October 2009

% updates the DataSetInventory structures
% without arguments for all the following DataSetBranches
if nargin < 1 
DSI_OceanWind_QuikSCAT
DSI_OceanWind_SeaWinds
DSI_OceanWind_CCMP
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% with one argument for the specified DataSetBranch only     
eval(varargin{1}); 
end

% a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCLAT URLCLON URLCDEPTH])
% constraints URLCTIME URLCLAT URLCLON URLCDEPTH can be in different order
% constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DSI=DSI_OceanWind_QuikSCAT 
DSI.DataSetName='OceanWind';
DSI.DataSetBranch='OceanWind_QuikSCAT';
% Example of request in the dataset
%http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data/2000/QS_XWGRD3_2000001.20070371658.gz?
%asc_avg_wind_speed[0:1:719][0:1:1439],
%des_avg_wind_speed[0:1:719][0:1:1439],
%asc_avg_wind_vel_u[0:1:719][0:1:1439],
%des_avg_wind_vel_u[0:1:719][0:1:1439],
%asc_avg_wind_vel_v[0:1:719][0:1:1439],
%des_avg_wind_vel_v[0:1:719][0:1:1439],
%asc_avg_wind_speed_sq[0:1:719][0:1:1439],
%des_avg_wind_speed_sq[0:1:719][0:1:1439],
%asc_wvc_count[0:1:719][0:1:1439],
%des_wvc_count[0:1:719][0:1:1439],
%asc_time_frac[0:1:719][0:1:1439],
%des_time_frac[0:1:719][0:1:1439],
%asc_rain_prob[0:1:719][0:1:1439],
%des_rain_prob[0:1:719][0:1:1439],
%asc_rain_flag[0:1:719][0:1:1439],
%des_rain_flag[0:1:719][0:1:1439]

DSI.URLSITE='http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data';
DSI.URLPATH='URLPATH=[''/'' YYYY ''/''];';
%DSI.URLFILE='FN=[''QS_XWGRD3_'' YYYY DDD ''.'']; EXT=''.gz'';URLFILE=htmlfilename(FN,EXT)'; 
%DSI.URLFILE='FN=[''QS_XWGRD3_'' YYYY DDD ''.'']; EXT=''.gz''; URLFILE=htmlfilename([URLSITE URLPATH],FN,EXT)';
DSI.URLFILE='FN=[''QS_XWGRD3_'' YYYY DDD ''.''];EXT=''.gz'';URL=[URLSITE URLPATH];S=urlread(URL);S=regexp(S,[FN ''.*?'' EXT],''match'');URLFILE=S{1};'
DSI.CTIME='';
DSI.PASS='if mod(Time,1)<0.5 PASS=''asc''; else PASS=''des''; end';
%DSI.CVAR='if mod(Time,1)<0.5 PASS=''asc''; else PASS=''des''; end; CVAR=[PASS ''_'' DSI.Fields_NameDS{kV}];';
%DSI.FIELDR='if mod(Time,1)<0.5 PASS=''asc''; else PASS=''des''; end; FIELDR=[PASS ''_'' DSI.Fields_NameReturned{kV}];';
% shorter version, PASS is evaluated 
DSI.CVAR='CVAR=[PASS ''_'' DSI.Fields_NameDS{j}];';
DSI.FIELDR='FIELDR=[PASS ''_'' DSI.Fields_NameReturned{j}];';
DSI.FIELDDS='FIELDDS=[PASS ''_'' DSI.Fields_NameDS{j}];';

LL={'latitude','longitude'};

DSI.FieldsTable={
%    opendap           long_name           name in            returned name       menu name     opendap  dataset  checkbox Units
%    name                                 in dataset                                             coord   coord    number
    'wind_speed'    ,'avg_wind_speed'   ,'avg_wind_speed'   ,'avg_wind_speed'   ,'wind speed'      ,LL    ,LL     ,103  ,'m/s'    ,'average wind speed, m/s';                
    'wind_vel_u'    ,'avg_wind_vel_u'   ,'avg_wind_vel_u'   ,'avg_wind_vel_u'   ,'wind vel u'      ,LL    ,LL     ,101  ,'m/s'    ,'average wind velocity towards East, m/s';
    'wind_vel_v'    ,'avg_wind_vel_v'   ,'avg_wind_vel_v'   ,'avg_wind_vel_v'   ,'wind vel v'      ,LL    ,LL     ,102  ,'m/s'    ,'average wind velocity towards North, m/s';
    'wind_speed_sq' ,'avg_wind_speed_sq','avg_wind_speed_sq','avg_wind_speed_sq','wind speed sq'   ,LL    ,LL     ,109  ,'m^2/s^2','average wind speed squared, m^2/s^2';
    'wvc_count'     ,'wvc_count'        ,'wvc_count'        ,'wvc_count'        ,'wvc count'       ,LL    ,LL     ,104  ,''       ,'wind velocity cell count';        
    'time_frac'     ,'time_frac'        ,'time_frac'        ,'time_frac'        ,'time of day'     ,LL    ,LL     ,105  ,'day'    ,'time of day, fractional day';
    'rain_prob'     ,'rain_prob'        ,'rain_prob'        ,'rain_prob'        ,'rain probability',LL    ,LL     ,106  ,''       ,'rain prob';
    'rain_flag'     ,'rain_flag'        ,'rain_flag'        ,'rain_flag'        ,'rain flag'       ,LL    ,LL     ,107  ,''       ,'rain flag';        
     };
 
DSI.Fields              =DSI.FieldsTable(:,1);
DSI.Fields_NameLong     =DSI.FieldsTable(:,2);
DSI.Fields_NameDS       =DSI.FieldsTable(:,3);
DSI.Fields_NameReturned =DSI.FieldsTable(:,4);
DSI.Fields_NameMenu     =DSI.FieldsTable(:,5);
DSI.Fields_Coord        =DSI.FieldsTable(:,6);
DSI.Fields_CoordDS      =DSI.FieldsTable(:,7);
DSI.Fields_CBNum        =DSI.FieldsTable(:,8);
DSI.Fields_Units        =DSI.FieldsTable(:,9);
DSI.Fields_ToolTip      =DSI.FieldsTable(:,10);

DSI.Range.Time1=datenum(1999,01,200);
DSI.Range.Time2=now; 
DSI.Time=(DSI.Range.Time1+6/24:0.5:DSI.Range.Time2+18/24)';
%6am LST ascending pass; 6pm LST descending pass
DSI.TimesPerDay=2;
DSI.Range.TRes='daily'
DSI.Range.HRes='1/4 deg, 25km'

NY=720;NX=1440;
DSI.Latitude = 180/NY*((1:1:NY)-0.5)'-90;
DSI.Longitude= 360/NX*((1:1:NX)-0.5)';

DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);

% set FormulaNaN and FormulaCnv to standard form

% need to eval in get program because FIELDDS depends on pass
for j=1:length(DSI.Fields)
    FIELD=DSI.Fields{j};
    FormulaNaN='eval([''f(find(f==A.'' FIELDDS ''.ml__FillValue))=NaN;'']);';
    eval(['DSI.FormulaNaN.' FIELD '='''';']);
    FormulaCnv='eval([''f=A.'' FIELDDS ''.scale_factor*f+A.'' FIELDDS ''.add_offset;'']);';
    eval(['DSI.FormulaCnv.' FIELD '=FormulaCnv;']);
end
% if they are non standard change them in the calling function

%A=loaddap('-A','http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data/2007/QS_XWGRD3_2007033.20070371630.gz?')
%A = 
%       asc_avg_wind_speed: [1x1 struct]
%       des_avg_wind_speed: [1x1 struct]
%       asc_avg_wind_vel_u: [1x1 struct]
%       des_avg_wind_vel_u: [1x1 struct]
%       asc_avg_wind_vel_v: [1x1 struct]
%       des_avg_wind_vel_v: [1x1 struct]
%    asc_avg_wind_speed_sq: [1x1 struct]
%    des_avg_wind_speed_sq: [1x1 struct]
%            asc_wvc_count: [1x1 struct]
%            des_wvc_count: [1x1 struct]
%            asc_time_frac: [1x1 struct]
%            des_time_frac: [1x1 struct]
%            asc_rain_prob: [1x1 struct]
%            des_rain_prob: [1x1 struct]
%            asc_rain_flag: [1x1 struct]
%            des_rain_flag: [1x1 struct]
%        Global_Attributes: [1x1 struct]
%A.asc_avg_wind_speed
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_avg_wind_speed"'
%                units: '"m/s"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_avg_wind_speed'
%A.des_avg_wind_speed
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_avg_wind_speed"'
%                units: '"m/s"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_avg_wind_speed'
%A.asc_avg_wind_vel_u
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_avg_wind_vel_u"'
%                units: '"m/s"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_avg_wind_vel_u'
%A.des_avg_wind_vel_u
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_avg_wind_vel_u"'
%                units: '"m/s"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_avg_wind_vel_u'
%A.asc_avg_wind_vel_v
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_avg_wind_vel_v"'
%                units: '"m/s"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_avg_wind_vel_v'
%A.des_avg_wind_vel_v
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_avg_wind_vel_v"'
%                units: '"m/s"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_avg_wind_vel_v'
%A.asc_avg_wind_speed_sq
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_avg_wind_speed_sq"'
%                units: '"m^2/s^2"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_avg_wind_speed_sq'
%A.des_avg_wind_speed_sq
%ans = 
%         scale_factor: 0.0100
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_avg_wind_speed_sq"'
%                units: '"m^2/s^2"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_avg_wind_speed_sq'
%A.asc_wvc_count
%ans = 
%         scale_factor: 1
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_wvc_count"'
%                units: '"counts"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_wvc_count'
%A.des_wvc_count
%ans = 
%         scale_factor: 1
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_wvc_count"'
%                units: '"counts"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_wvc_count'
%A.asc_time_frac
%ans = 
%         scale_factor: 2.0000e-005
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_time_frac"'
%                units: '"day"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_time_frac'
%A.des_time_frac
%ans = 
%         scale_factor: 2.0000e-005
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_time_frac"'
%                units: '"day"'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_time_frac'
%A.asc_rain_prob
%ans = 
%         scale_factor: 1.0000e-003
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_rain_prob"'
%                units: '" "'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_rain_prob'
%A.des_rain_prob
%ans = 
%         scale_factor: 1.0000e-003
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_rain_prob"'
%                units: '" "'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_rain_prob'
%A.asc_rain_flag
%ans = 
%         scale_factor: 1
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"asc_rain_flag"'
%                units: '" "'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'asc_rain_flag'
%A.des_rain_flag
%ans = 
%         scale_factor: 1
%     scale_factor_err: 0
%           add_offset: 0
%       add_offset_err: 0
%        calibrated_nt: 22
%            long_name: '"des_rain_flag"'
%                units: '" "'
%         DODS_ML_Size: [2x1 double]
%    DODS_ML_Real_Name: 'des_rain_flag'


S1=['time in seconds since 1970-01-01 00:00:00;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units.'];
DSI.Readme=[S1 '/' S2 '/' S3];

DSI=DSI_common3(DSI);
SaveDSI(DSI)

function SaveDSI(DSI)
eval(['DSI_' DSI.DataSetBranch '=DSI;'])
dsi_filename = 'dsi_oceanwind.mat';
%if (~isdeployed)
    pname = mfilename('fullpath');
    pname = pname(1:length(pname) - length(mfilename));
    dsi_filename = [pname,dsi_filename];
%end
if ~exist(dsi_filename,'file')
save(dsi_filename,['DSI_' DSI.DataSetBranch])
end
save(dsi_filename,['DSI_' DSI.DataSetBranch],'-append')

function DSI=DSI_common3(DSI)
CoordinatesTable={
'latitude','latitude','degrees_north';
'longitude','longitude','degrees_east';
%'depth','depth','m';
'time','time','days since 0000-01-01 00:00:00';
};

DSI.Coordinates=CoordinatesTable(:,1);
DSI.Dimensions.latitude=length(DSI.Latitude);
DSI.Dimensions.longitude=length(DSI.Longitude);
%DSI.Dimensions.depth=length(DSI.Depth);
DSI.Dimensions.time=length(DSI.Time);
for k=1:length(DSI.Fields)
    FIELD=DSI.Fields{k};
eval(['DSI.Variables.' FIELD '=DSI.Fields_Coord{k};'])
end
DSI.Variables.latitude={'latitude'};
DSI.Variables.longitude={'longitude'};
%DSI.Variables.depth={'depth'};
DSI.Variables.time={'time'};

for k=1:length(DSI.Fields)
    FIELD=DSI.Fields{k};
eval(['DSI.Attributes.' FIELD '.long_name=DSI.Fields_NameLong{k};']);
eval(['DSI.Attributes.' FIELD '.units    =DSI.Fields_Units{k};']);
end
for k=1:length(DSI.Coordinates)
    COORD=DSI.Coordinates{k};
eval(['DSI.Attributes.' COORD '.long_name=CoordinatesTable{k,2};']);
eval(['DSI.Attributes.' COORD '.units    =CoordinatesTable{k,3};']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DSI=DSI_OceanWind_SeaWinds 
DSI.DataSetName='OceanWind';
DSI.DataSetBranch='OceanWind_SeaWinds';

% A value of 0 for the third index indicates the ascending phase of the orbit while a value of 1 indicates the descending phase of the orbit.
%The SeaWinds on SeaWinds Level 3 data set consists of gridded values of scalar wind speed,
%meridional and zonal components of wind velocity and time given in fraction of a day. Rain
%probability determined using the Impact Multidimensional Histogram (IMUDH) Rain Flagging
%technique is also included as an indicator of wind values that may have degraded accuracy due to
%the presence of rain.
%The Level 3 data were obtained from the Direction Interval Retrieval with Threshold Nudging
%(DIRTH) wind vector solutions contained in the SeaWinds Level 2B data and are provided on an
%approximately 0.25 x 0.25 global grid. Separate maps are provided for both the ascending pass
%(10:30PM LST equator crossing) and descending pass (10:30AM LST equator crossing). By
%maintaining the data at nearly the original Level 2B sampling resolution and separating the
%ascending and descending passes, very little overlap occurs in one day. However, when overlap
%between subsequent swaths does occur, the values are over-written, not averaged. Therefore, a
%SeaWinds on ADEOS II Level 3 file contains only the latest measurement for each day.

% Example of request in the dataset
%http://dods.jpl.nasa.gov/opendap/ocean_wind/seawinds/L3/data/2003/SW_S3S_2003100.20053531747.gz?
%rep_wind_speed[0:1:719][0:1:1439][0:1:1],
%rep_wind_velocity_u[0:1:719][0:1:1439][0:1:1],
%rep_wind_velocity_v[0:1:719][0:1:1439][0:1:1],
%rep_atten_corr[0:1:719][0:1:1439][0:1:1],
%rep_time_of_day[0:1:719][0:1:1439][0:1:1],
%rep_rain_probability[0:1:719][0:1:1439][0:1:1],
%rep_srad_rain_rate[0:1:719][0:1:1439][0:1:1],
%rep_amsr_rain_indicator[0:1:719][0:1:1439][0:1:1],
%rain_flag[0:1:719][0:1:1439][0:1:1],
%null_data_indicator[0:1:719][0:1:1439][0:1:1],
%grid_cell_quality_flag[0:1:719][0:1:1439][0:1:1]

DSI.LOADDAPURL='LOADDAPURL=[URLSITE URLPATH URLFILE ''?'' CVAR R.CLAT R.CLON CTIME];';

DSI.URLSITE='http://dods.jpl.nasa.gov/opendap/ocean_wind/seawinds/L3/data';
DSI.URLPATH='URLPATH=[''/'' YYYY ''/''];';
%DSI.URLFILE='FN=[''QS_XWGRD3_'' YYYY DDD ''.'']; EXT=''.gz'';URLFILE=htmlfilename(FN,EXT)'; 
%DSI.URLFILE='FN=[''QS_XWGRD3_'' YYYY DDD ''.'']; EXT=''.gz''; URLFILE=htmlfilename([URLSITE URLPATH],FN,EXT)';
DSI.URLFILE='FN=[''SW_S3S_'' YYYY DDD ''.''];EXT=''.gz'';URL=[URLSITE URLPATH];S=urlread(URL);S=regexp(S,[FN ''.*?'' EXT],''match'');URLFILE=S{1};'
DSI.CTIME='if mod(Time,1)<0.5 CTIME=''[0]''; else CTIME=''[1]''; end';
% shorter version, PASS is evaluated 
DSI.CVAR='CVAR=[DSI.Fields_NameDS{j}];';
DSI.FIELDR='FIELDR=[DSI.Fields_NameReturned{j}];';
DSI.FIELDDS='FIELDDS=[DSI.Fields_NameDS{j}];';

% define list of fields or variables available in the dataset
LL={'latitude','longitude'};

DSI.FieldsTable={
%    opendap                    long_name                 name in                   returned name         menu name            opendap  dataset  checkbox  Units
%    name                                                in dataset                                                            coord   coord    number
    'wind_speed'          ,'rep_wind_speed'         ,'rep_wind_speed'         ,'rep_wind_speed'         ,'wind speed'         ,LL    ,LL       ,103      ,'m/s';
    'wind_velocity_u'     ,'rep_wind_velocity_u'    ,'rep_wind_velocity_u'    ,'rep_wind_velocity_u'    ,'wind velocity u'    ,LL    ,LL       ,101      ,'m/s';                
    'wind_velocity_v'     ,'rep_wind_velocity_v'    ,'rep_wind_velocity_v'    ,'rep_wind_velocity_v'    ,'wind velocity v'    ,LL    ,LL       ,102      ,'m/s';
    'atten_corr'          ,'rep_atten_corr'         ,'rep_atten_corr'         ,'rep_atten_corr'         ,'atten corr'         ,LL    ,LL       ,104      ,'dB';
    'time_of_day'         ,'rep_time_of_day'        ,'rep_time_of_day'        ,'rep_time_of_day'        ,'time of day'        ,LL    ,LL       ,105      ,'n/a';
    'rain_probability'    ,'rep_rain_probability'   ,'rep_rain_probability'   ,'rep_rain_probability'   ,'rain probability'   ,LL    ,LL       ,106      ,'n/a';
    'srad_rain_rate'      ,'rep_srad_rain_rate'     ,'rep_srad_rain_rate'     ,'rep_srad_rain_rate'     ,'srad rain rate'     ,LL    ,LL       ,107      ,'km*mm/hr';
    'amsr_rain_indicator' ,'rep_amsr_rain_indicator','rep_amsr_rain_indicator','rep_amsr_rain_indicator','samsr rain indicator',LL    ,LL       ,108      ,'n/a';
    'rain_flag'              ,'rain_flag'              ,'rain_flag'              ,'rain_flag'              ,'rain flag'              ,LL    ,LL       ,109      ,'n/a';
    'null_data_indicator'    ,'null_data_indicator'    ,'null_data_indicator'    ,'null_data_indicator'    ,'null data indicator'    ,LL    ,LL       ,110      ,'n/a';
    'grid_cell_quality_flag' ,'grid_cell_quality_flag' ,'grid_cell_quality_flag' ,'grid_cell_quality_flag' ,'grid cell quality flag' ,LL    ,LL       ,111      ,'n/a';
    };
DSI.Fields              =DSI.FieldsTable(:,1);
DSI.Fields_NameLong     =DSI.FieldsTable(:,2);
DSI.Fields_NameDS       =DSI.FieldsTable(:,3);
DSI.Fields_NameReturned =DSI.FieldsTable(:,4);
DSI.Fields_NameMenu     =DSI.FieldsTable(:,5);
DSI.Fields_Coord        =DSI.FieldsTable(:,6);
DSI.Fields_CoordDS      =DSI.FieldsTable(:,7);
DSI.Fields_CBNum        =DSI.FieldsTable(:,8);
DSI.Fields_Units        =DSI.FieldsTable(:,9);
DSI.Fields_ToolTip      =DSI.FieldsTable(:,2);

DSI.Range.Time1=datenum(2003,01,100);
DSI.Range.Time2=datenum(2003,01,297); 
DSI.Time=(DSI.Range.Time1+3/24:0.5:DSI.Range.Time2+15/24)';
%6am LST ascending pass; 6pm LST descending pass ?
DSI.TimesPerDay=2;
DSI.Range.TRes='daily';
DSI.Range.HRes='1/4 deg, 25km';

NY=720;NX=1440;
DSI.Latitude = 180/NY*((1:1:NY)-0.5)'-90;
DSI.Longitude= 360/NX*((1:1:NX)-0.5)';

DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);

% set FormulaNaN and FormulaCnv to standard form
for j=1:length(DSI.Fields)
    FIELD=DSI.Fields{j};
    FIELDDS=DSI.Fields_NameDS{j};
%    FormulaNaN='[''f(find(f==A.'' FIELDDS ''.ml__FillValue))=NaN;''];';
%    eval(['DSI.FormulaNaN.' FIELD '=''f(find(f==A.' FIELDDS '.ml__FillValue))=NaN;'';']);
%    eval(['DSI.FormulaNaN.' FIELD '=FormulaNaN;']);
    eval(['DSI.FormulaNaN.' FIELD '='''';']);
%    FormulaCnv='eval([''f=A.'' FIELDDS ''.scale_factor*f+A.'' FIELDDS ''.add_offset;'']);';
    FormulaCnv=['f=A.' FIELDDS '.scale_factor*f+A.' FIELDDS '.add_offset;'];
    eval(['DSI.FormulaCnv.' FIELD '=FormulaCnv;']);
end

S1=['time in seconds since 1970-01-01 00:00:00;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units.'];
DSI.Readme=[S1 '/' S2 '/' S3];
DSI=DSI_common3(DSI);
SaveDSI(DSI)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DSI=DSI_OceanWind_CCMP 
DSI.DataSetName='OceanWind';
DSI.DataSetBranch='OceanWind_CCMP';
DSI.DataSetBranch_long_name='Ocean Wind Cross-Calibrated Multi-Platform data product processing Level 3.0';

%http://dods.jpl.nasa.gov/opendap/ocean_wind/ccmp/L3.0/data/flk/1987/M07/README.txt: 
%This directory contains all of the Level 3.0 daily NetCDF data granules.
%
%Each file uses the following naming convention:
%
%NAME_YYYYMMDD_vVVlLLPPP.TYPE.gz
%
%NAME     = Unique Data Descriptor
%YYYYMMDD = Calendar Date expressed as Year(YYYY), Month(MM), Day(DD).
%VV	 = Two-Digit Product Version Number (Divide by 10 to get actual number)
%LL	 = Two-Digit Product Level (Divide by 10 to get actual product level)
%PPP	 = Product Processing Stream (i.e., flk or llk)
%TYPE	 = File Type Designating the Readable Data Format (i.e., nc = NetCDF)
%
%If you have any questions or comments, please email us at
%podaac@podaac.jpl.nasa.gov.
%
%Last updated: 4 May 2009

% Example of loaddap request:
%http://dods.jpl.nasa.gov/opendap/ocean_wind/ccmp/L3.0/data/flk/1987/M07/
%analysis_19870701_v11l30flk.nc.gz?lon[0:1:1439],lat[0:1:627],time[0:1:3],
%uwnd[0:1:3][0:1:627][0:1:1439],vwnd[0:1:3][0:1:627][0:1:1439],nobs[0:1:3][0:1:627][0:1:1439]
%
% http://dods.jpl.nasa.gov/opendap/ocean_wind/ccmp/L3.0/data/flk/2008/M12/
%analysis_20081230_v11l30flk.nc.gz?lon[0:1:1439],lat[0:1:627],time[0:1:3],
%uwnd[0:1:3][0:1:627][0:1:1439],vwnd[0:1:3][0:1:627][0:1:1439],nobs[0:1:3][0:1:627][0:1:1439]
%
DSI.URLSITE='http://dods.jpl.nasa.gov/opendap/ocean_wind/ccmp/L3.0/data/flk';
DSI.URLPATH='YYYY=datestr(Time,''yyyy''); MM=datestr(Time,''mm''); URLPATH=[''/'' YYYY ''/M'' MM ''/''];';
DSI.URLFILE='YYYYMMDD=datestr(Time,''yyyymmdd''); URLFILE=[''analysis_'' YYYYMMDD ''_v11l30flk.nc.gz''];';
% depending on hour of day, HH: 00,06,12,18, CTIME: [0],[1],[2],[3] 
DSI.CTIME='HH=datestr(Time,''HH''); CTIME=[ ''['' num2str(str2num(HH)/6) '']'' ];';

DSI.Range.Time1=datenum(1987,07,01);
DSI.Range.Time2=datenum(2008,12,31); 
% Four times a day: 0,6,12,18h
DSI.Time=(DSI.Range.Time1 : 0.25 : DSI.Range.Time2+18/24)';
DSI.TimesPerDay=4;
NY=628;NX=1440;
DSI.Latitude = 180/NY*((1:1:NY)-0.5)'-90;
DSI.Longitude= 360/NX*((1:1:NX)-0.5)';
DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);
DSI.Range.TRes='daily'
DSI.Range.HRes='1/4 deg, 25km'

DSI.LOADDAPURL='LOADDAPURL=[URLSITE URLPATH URLFILE ''?'' CVAR CTIME R.CLAT R.CLON];';

LatLon={'latitude','longitude'};
DSI.FieldsTable={
%opendap    long_name            name in    returned      menu                    opendap  dataset  checkbox  Units
%name                            in dataset name          name                    coord    coord    number
'uwnd' ,'u-wind at 10 meters'   ,'uwnd'    ,'uwnd.uwnd' ,'u wind'                 ,LatLon, LatLon,    101,    'm/s',   'u-wind at 10 meters, m/s'; 
'vwnd' ,'v-wind at 10 meters'   ,'vwnd'    ,'vwnd.vwnd' ,'v wind'                 ,LatLon, LatLon,    102,    'm/s',   'v-wind at 10 meters, m/s'; 
'nobs' ,'number of observations','nobs'    ,'nobs.nobs' ,'number of observations' ,LatLon, LatLon,    104,    'count', 'number of observations, count'; 
};
DSI.Fields              =DSI.FieldsTable(:,1);
DSI.Fields_NameLong     =DSI.FieldsTable(:,2);
DSI.Fields_NameDS       =DSI.FieldsTable(:,3);
DSI.Fields_NameReturned =DSI.FieldsTable(:,4);
DSI.Fields_NameMenu     =DSI.FieldsTable(:,5);
DSI.Fields_Coord        =DSI.FieldsTable(:,6);
DSI.Fields_CoordDS      =DSI.FieldsTable(:,7);
DSI.Fields_CBNum        =DSI.FieldsTable(:,8);
DSI.Fields_Units        =DSI.FieldsTable(:,9);
DSI.Fields_ToolTip      =DSI.FieldsTable(:,10);

% set FormulaNaN and FormulaCnv to standard form
for j=1:length(DSI.Fields)
    FIELD=DSI.Fields{j};
    FIELDDS=DSI.Fields_NameDS{j};
    eval(['DSI.FormulaNaN.' FIELD '=''f(find(f==A.' FIELDDS '.missing_value))=NaN;'';']);
    eval(['DSI.FormulaCnv.' FIELD '=''f=A.' FIELDDS '.scale_factor*f+A.' FIELDDS '.add_offset;'';']);
end
% if they are non standard change them in the calling function

CoordinatesTable={
'latitude','latitude','degrees_north';
'longitude','longitude','degrees_east';
%'depth','depth','m';
'time','time','days since 0000-01-01 00:00:00';
};

DSI.Coordinates=CoordinatesTable(:,1);
DSI.Dimensions.latitude=length(DSI.Latitude);
DSI.Dimensions.longitude=length(DSI.Longitude);
%DSI.Dimensions.depth=length(DSI.Depth);
DSI.Dimensions.time=length(DSI.Time);
for k=1:length(DSI.Fields)
    FIELD=DSI.Fields{k};
eval(['DSI.Variables.' FIELD '=DSI.Fields_Coord{k};'])
end
DSI.Variables.latitude={'latitude'};
DSI.Variables.longitude={'longitude'};
%DSI.Variables.depth={'depth'};
DSI.Variables.time={'time'};

for k=1:length(DSI.Fields)
    FIELD=DSI.Fields{k};
eval(['DSI.Attributes.' FIELD '.long_name=DSI.Fields_NameLong{k};']);
eval(['DSI.Attributes.' FIELD '.units    =DSI.Fields_Units{k};']);
end
for k=1:length(DSI.Coordinates)
    COORD=DSI.Coordinates{k};
eval(['DSI.Attributes.' COORD '.long_name=CoordinatesTable{k,2};']);
eval(['DSI.Attributes.' COORD '.units    =CoordinatesTable{k,3};']);
end

%A=loaddap('-A','http://dods.jpl.nasa.gov/opendap/ocean_wind/ccmp/L3.0/data/flk/2008/M01/analysis_20080101_v11l30flk.nc.gz?lon[0:1:1439],lat[0:1:627],time[0:1:3],uwnd[0:1:3][0:1:627][0:1:1439],vwnd[0:1:3][0:1:627][0:1:1439],nobs[0:1:3][0:1:627][0:1:1439]')
%
%A = 
%                  lon: [1x1 struct]
%                  lat: [1x1 struct]
%                 time: [1x1 struct]
%                 uwnd: [1x1 struct]
%                 vwnd: [1x1 struct]
%                 nobs: [1x1 struct]
%    Global_Attributes: [1x1 struct]
%
%A.lon
%
%ans = 
%                units: '"degrees_east"'
%            long_name: '"Longitude"'
%         actual_range: [2x1 double]
%         DODS_ML_Size: 1440
%    DODS_ML_Real_Name: 'lon'
%
%A.lat
%
%ans = 
%                units: '"degrees_north"'
%            long_name: '"Latitude"'
%         actual_range: [2x1 double]
%         DODS_ML_Size: 628
%    DODS_ML_Real_Name: 'lat'
%
%A.time
%
%ans = 
%                units: '"hours since 1987-01-01 00:00:0.0"'
%            long_name: '"Time"'
%         actual_range: [2x1 double]
%              delta_t: '"0000-00-00 06:00:00"'
%           avg_period: '""'
%         DODS_ML_Size: 4
%    DODS_ML_Real_Name: 'time'
%
%A.uwnd
%
%ans = 
%            long_name: '"u-wind at 10 meters"'
%                units: '"m/s"'
%           add_offset: 0
%         scale_factor: 0.0031
%          valid_range: [2x1 double]
%         actual_range: [2x1 double]
%        missing_value: -32767
%    DODS_ML_Real_Name: 'uwnd'
%                 uwnd: [1x1 struct]
%                 time: [1x1 struct]
%                  lat: [1x1 struct]
%                  lon: [1x1 struct]
%
%A.vwnd
%
%ans = 
%            long_name: '"v-wind at 10 meters"'
%                units: '"m/s"'
%           add_offset: 0
%         scale_factor: 0.0031
%          valid_range: [2x1 double]
%         actual_range: [2x1 double]
%        missing_value: -32767
%    DODS_ML_Real_Name: 'vwnd'
%                 vwnd: [1x1 struct]
%                 time: [1x1 struct]
%                  lat: [1x1 struct]
%                  lon: [1x1 struct]
%
%A.nobs
%
%ans = 
%            long_name: '"number of observations"'
%                units: '"count"'
%           add_offset: 32766
%         scale_factor: 1
%          valid_range: [2x1 double]
%         actual_range: [2x1 double]
%        missing_value: -32767
%    DODS_ML_Real_Name: 'nobs'
%                 nobs: [1x1 struct]
%                 time: [1x1 struct]
%                  lat: [1x1 struct]
%                  lon: [1x1 struct]
%
%A.Global_Attributes
%
%ans = 
%    DODS_ML_Real_Name: 'Global_Attributes'
%            NC_GLOBAL: [1x1 struct]
%           DODS_EXTRA: [1x1 struct]
%

S1=['time in seconds since 1970-01-01 00:00:00;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units.'];
DSI.Readme=[S1 '/' S2 '/' S3];
SaveDSI(DSI)

